
### Vectorisation

# create data frame
N <- 10000
x1 <- runif(N)
x2 <- runif(N)
d <- data.frame(x1, x2)

# basic loop
system.time(for(i in 1:nrow(d)) {
    d$mean1[i] <- mean(c(d$x1[i], d$x2[i]))
})

# apply type
system.time(d$mean2 <- apply(d, 1, mean))

# proper vectorisation
system.time(d$mean3 <- rowMeans(d))

# check results
all.equal(d$mean1, d$mean2)
all.equal(d$mean1, d$mean3)

# other example
x <- c()
system.time(for (i in 1:1e+05) {
    x <- c(x, i)
})
system.time(y <- seq(1, 1e+05))

### Gibbs sampler

# R version
gibbs_r <- function(N, thin) {
    mat <- matrix(nrow = N, ncol = 2)
    x <- y <- 0
    
    for (i in 1:N) {
        for (j in 1:thin) {
            x <- rgamma(1, 3, 1 / (y * y + 4))
            y <- rnorm(1, 1 / (x + 1), 1 / sqrt(2 * (x + 1)))
        }
        mat[i, ] <- c(x, y)
    }
    mat
}

# Load the Rcpp version
# install.packages('Rcpp')
library(Rcpp)
sourceCpp('rcpp.cpp')

# About 25x faster on my machine
system.time(res1 <- gibbs_r(10000, 10))
system.time(res2 <- gibbs_cpp(10000, 10))

### Summary

# R function + loop
vacc1a <- function(age, female, ily) {
    p <- 0.25 + 0.3 * 1 / (1 - exp(0.04 * age)) + 0.1 * ily
    p <- p * if (female) 1.25 else 0.75
    p <- max(0, p)
    p <- min(1, p)
    p
}
vacc1 <- function(age, female, ily) {
    n <- length(age)
    out <- numeric(n)
    for (i in seq_len(n)) {
        out[i] <- vacc1a(age[i], female[i], ily[i])
    }
    out
}

# vectorised
vacc2 <- function(age, female, ily) {
    p <- 0.25 + 0.3 * 1 / (1 - exp(0.04 * age)) + 0.1 * ily
    p <- p * ifelse(female, 1.25, 0.75)
    p <- pmax(0, p)
    p <- pmin(1, p)
    p
}

# generate sample data
n <- 1000000
age <- rnorm(n, mean = 50, sd = 10)
female <- sample(c(T, F), n, rep = TRUE)
ily <- sample(c(T, F), n, prob = c(0.8, 0.2), rep = TRUE)

# mendatory checks
all.equal(vacc1(age, female, ily), vacc2(age, female, ily))
all.equal(vacc1(age, female, ily), vacc3(age, female, ily))

# rule of 10
system.time(vacc1(age, female, ily))
system.time(vacc2(age, female, ily))
system.time(vacc3(age, female, ily))
